/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wTimers.h"
#include <iomanip>

using namespace fwk;

Timers::Timers()
{
}

Timer &
Timers::operator[](std::string const &name)
{
    return timers[name];
}

void Timers::reset()
{
    for (auto it = timers.begin(); it != timers.end(); ++it)
        it->second.reset();
}

void Timers::write(std::ostream &out) const
{
    out << std::fixed << std::setprecision(1);
    out << std::setw(15) << std::left << " "
        << std::setw(10) << std::right << "user"
        << std::setw(10) << std::right << "real"
        << std::setw(10) << std::right << "kernel"
        << '\n';

    for (auto it = timers.begin(); it != timers.end(); ++it)
    {
        Times tm = it->second.read();
        out << std::setw(15) << std::left << it->first
            << std::setw(10) << std::right << tm.getUser().sec()
            << std::setw(10) << std::right << tm.getReal().sec()
            << std::setw(10) << std::right << tm.getKernel().sec()
            << '\n';
    }
}
