SetFactory("OpenCASCADE");
If(GMSH_MAJOR_VERSION<=3 && GMSH_MINOR_VERSION<=0 && GMSH_PATCH_VERSION<6)
  Printf("gmsh %g.%g.%g is not supported by this .geo",GMSH_MAJOR_VERSION,GMSH_MINOR_VERSION,GMSH_PATCH_VERSION);
Else
  LX = 10;
  LY = 10;
  LZ = 1;

  NX = 4;
  NY = 4;

  CX = LX/2;
  CY = LY/2;

  R = 2;
  dR = 1;

  nT = 20;
  nZ = 3;
  ndR = 10;

  PI = Acos (-1.);

  lc = 1;

  Point(201)  = { CX,    CY,  0.,     lc};

  Point(202)  = { CX+(R+dR)*Cos(1*PI/4),  CY+(R+dR)*Sin(1*PI/4), 0.,     lc};
  Point(203)  = { CX+(R+dR)*Cos(3*PI/4),  CY+(R+dR)*Sin(3*PI/4), 0.,     lc};
  Point(204)  = { CX+(R+dR)*Cos(5*PI/4),  CY+(R+dR)*Sin(5*PI/4), 0.,     lc};
  Point(205)  = { CX+(R+dR)*Cos(7*PI/4),  CY+(R+dR)*Sin(7*PI/4), 0.,     lc};

  Point(206)  = { LX,  LY, 0.,     lc};
  Point(207)  = { 0.,  LY, 0.,     lc};
  Point(208)  = { 0.,  0., 0.,     lc};
  Point(209)  = { LX,  0., 0.,     lc};

  Circle(2101) = {202, 201, 203};
  Circle(2102) = {203, 201, 204};
  Circle(2103) = {204, 201, 205};
  Circle(2104) = {205, 201, 202};

  Line(2105) = {206, 207};
  Line(2106) = {207, 208};
  Line(2107) = {208, 209};
  Line(2108) = {209, 206};

  Line(2201) = {202, 206};
  Line(2202) = {203, 207};
  Line(2203) = {204, 208};
  Line(2204) = {205, 209};

  Point(301)  = { -NX*LX,  NY*LY+LY, 0.,     lc};
  Point(302)  = { 0,  NY*LY+LY, 0.,     lc};
  Point(303)  = { -NX*LX,  LY, 0.,     lc};

  Line(311) = {301, 302};
  Line(312) = {302, 207};
  Line(313) = {207, 303};
  Line(314) = {303, 301};

  Point(501)  = { LX,  NY*LY+LY, 0.,     lc};
  Point(502)  = { LX+NX*LX,  NY*LY+LY, 0.,     lc};
  Point(503)  = { LX+NX*LX,  LY, 0.,     lc};

  Line(511) = {501, 502};
  Line(512) = {502, 503};
  Line(513) = {503, 206};
  Line(514) = {206, 501};

  Point(801)  = { -NX*LX,  0, 0.,     lc};
  Point(802)  = { 0,  -NY*LY, 0.,     lc};
  Point(803)  = { -NX*LX,  -NY*LY, 0.,     lc};

  Line(811) = {801, 208};
  Line(812) = {208, 802};
  Line(813) = {802, 803};
  Line(814) = {803, 801};

  Point(1001)  = { LX+NX*LX,  0, 0.,     lc};
  Point(1002)  = { LX+NX*LX,  -NY*LY, 0.,     lc};
  Point(1003)  = { LX,  -NY*LY, 0.,     lc};

  Line(1011) = {209, 1001};
  Line(1012) = {1001, 1002};
  Line(1013) = {1002, 1003};
  Line(1014) = {1003, 209};

  Line(411) = {302, 501};
  Line(611) = {801, 303};
  Line(711) = {503, 1001};
  Line(911) = {1003, 802};

  Transfinite Line {2101, 2102, 2103, 2104, 2105, 2106, 2107, 2108, 411, 611, 711, 911} = nT Using Progression 1;
  Transfinite Line {2201, 2202, 2203, 2204} = ndR Using Progression 1;
  Transfinite Line {311,511,313,513,811,1011,813,1013} = NX*nT Using Progression 1;
  Transfinite Line {314,814,312,812,514,1014,512,1012} = NY*nT Using Progression 1;

  Line Loop(201) = {2101, 2202, -2105, -2201};
  Plane Surface(201) = {201};
  Transfinite Surface {201};
  Recombine Surface {201};

  Line Loop(202) = {2202, 2106, -2203, -2102};
  Plane Surface(202) = {202};
  Transfinite Surface {202};
  Recombine Surface {202};

  Line Loop(203) = {2103, 2204, -2107, -2203};
  Plane Surface(203) = {203};
  Transfinite Surface {203};
  Recombine Surface {203};

  Line Loop(204) = {2104, 2201, -2108, -2204};
  Plane Surface(204) = {204};
  Transfinite Surface {204};
  Recombine Surface {204};

  Line Loop(205) = {311, 312, 313, 314};
  Plane Surface(205) = {205};
  Transfinite Surface {205};
  Recombine Surface {205};

  Line Loop(206) = {511, 512, 513, 514};
  Plane Surface(206) = {206};
  Transfinite Surface {206};
  Recombine Surface {206};

  Line Loop(207) = {811, 812, 813, 814};
  Plane Surface(207) = {207};
  Transfinite Surface {207};
  Recombine Surface {207};

  Line Loop(208) = {1011, 1012, 1013, 1014};
  Plane Surface(208) = {208};
  Transfinite Surface {208};
  Recombine Surface {208};

  Line Loop(209) = {411, -514, 2105, -312};
  Plane Surface(209) = {209};
  Transfinite Surface {209};
  Recombine Surface {209};

  Line Loop(210) = {-313, 2106, -811, 611};
  Plane Surface(210) = {210};
  Transfinite Surface {210};
  Recombine Surface {210};

  Line Loop(1211) = {-513, 711, -1011, 2108};
  Plane Surface(1211) = {1211};
  Transfinite Surface {1211};
  Recombine Surface {1211};

  Line Loop(2212) = {2107, -1014, 911, -812};
  Plane Surface(2212) = {2212};
  Transfinite Surface {2212};
  Recombine Surface {2212};

  allSurfaces[] = Surface "*";
  Extrude {0, 0, LZ}
  {
    Surface{allSurfaces[]}; Layers{nZ}; Recombine;
  }

  Physical Volume("body") = {1,2,3,4,5,6,7,8,9,10,11,12};
  //+
  Physical Surface("outer part boundary") = {2226,2213,2220,2222};
  Physical Surface("load 1") = {2229,2234,2249};
  Physical Surface("load 2") = {2241,2246,2255};
  Physical Point("clamped xyz") = {1020};
  Physical Point("clamped xy") = {803};
  Physical Point("clamped y") = {1022};

EndIf
