#ifndef KATOPTRON_MORTAR_H
#define KATOPTRON_MORTAR_H

#include "Tpetra_Vector.hpp"
#include "Kokkos_ViewFactory.hpp"
#include "Tpetra_Map.hpp"
#include <mpi.h>
#include "Tpetra_CrsGraph.hpp"
#include <Epetra_Map.h>

#include "Tpetra_ConfigDefs.hpp"

#include "Tpetra_Map.hpp"
#include "Tpetra_MultiVector.hpp"
#include "Tpetra_Vector.hpp"
#include "Tpetra_CrsGraph.hpp"
#include "Tpetra_CrsMatrix.hpp"

#include "ElementsList.h"
#include "NodesList.h"
#include "ContactsList.h"
#include "Map.h"
#include "Matrices.h"

Teuchos::RCP<Tpetra::CrsMatrix<>> compute_B(Teuchos::RCP<katoptron::ElementsList> elementsList,
                                            Teuchos::RCP<katoptron::NodesList> nodesList,
                                            Teuchos::RCP<katoptron::ContactsList> contactsList,
                                            Teuchos::RCP<katoptron::Map> map, MPI_Comm comm, bool SignoriniCheck);

template <typename scalar>
void compute_B(Teuchos::RCP<katoptron::Matrices<scalar>> matrices,
               Teuchos::RCP<katoptron::ElementsList> elementsList,
               Teuchos::RCP<katoptron::NodesList> nodesList,
               Teuchos::RCP<katoptron::ContactsList> contactsList,
               Teuchos::RCP<katoptron::Map> map, MPI_Comm comm)
{
    Teuchos::RCP<Tpetra::CrsMatrix<>> B_double = compute_B(elementsList, nodesList, contactsList, map, comm, true);
    Teuchos::RCP<Tpetra::CrsMatrix<>> B_G_double;
    if (contactsList->isSignorini())
        B_G_double = compute_B(elementsList, nodesList, contactsList, map, comm, false);
    else
        B_G_double = B_double;

    typedef katoptron::Map::local_ordinal_type local_ordinal_type;
    typedef katoptron::Map::global_ordinal_type global_ordinal_type;

    size_t maxNumEntPerRow = B_G_double->getNodeMaxNumRowEntries();
    size_t maxNumEntPerRowT = 200; // Temporary fix

    matrices->B = rcp(new Tpetra::CrsMatrix<scalar, local_ordinal_type, global_ordinal_type>(map->mapLagrangeDofs, maxNumEntPerRow));
    matrices->B_2 = rcp(new Tpetra::CrsMatrix<scalar, local_ordinal_type, global_ordinal_type>(map->mapLagrangeDofs, maxNumEntPerRow));
    matrices->B_G = rcp(new Tpetra::CrsMatrix<scalar, local_ordinal_type, global_ordinal_type>(map->mapLagrangeDofs, maxNumEntPerRow));
    matrices->B_T = rcp(new Tpetra::CrsMatrix<scalar, local_ordinal_type, global_ordinal_type>(map->mapDofs, maxNumEntPerRowT));
    matrices->C = rcp(new Tpetra::CrsMatrix<scalar, local_ordinal_type, global_ordinal_type>(map->mapLagrangeDofs, 1));
    matrices->Cb = rcp(new Tpetra::CrsMatrix<scalar, local_ordinal_type, global_ordinal_type>(map->mapLagrangeDofs, 1));

    for (auto i = 0; i < map->mapLagrangeDofs->getNodeNumElements(); ++i)
    {
        if (map->mapLagrangeDofs->isNodeLocalElement(i))
        {
            local_ordinal_type local_index = i;
            global_ordinal_type global_index = map->mapLagrangeDofs->getGlobalElement(local_index);

            Teuchos::ArrayView<const local_ordinal_type> local_indices;
            Teuchos::ArrayView<const double> values;

            B_double->getLocalRowView(local_index, local_indices, values);

            for (auto k = 0; k < local_indices.size(); ++k)
            {
                global_ordinal_type column_index = B_double->getColMap()->getGlobalElement(local_indices[k]);

                matrices->B->insertGlobalValues(global_index,
                                                Teuchos::tuple<global_ordinal_type>(column_index),
                                                Teuchos::tuple<scalar>((scalar)values[k]));

                matrices->B_2->insertGlobalValues(global_index,
                                                  Teuchos::tuple<global_ordinal_type>(column_index),
                                                  Teuchos::tuple<scalar>((scalar)values[k]));

                matrices->B_T->insertGlobalValues(column_index,
                                                  Teuchos::tuple<global_ordinal_type>(global_index),
                                                  Teuchos::tuple<scalar>((scalar)values[k]));
            }

            B_G_double->getLocalRowView(local_index, local_indices, values);

            for (auto k = 0; k < local_indices.size(); ++k)
            {
                global_ordinal_type column_index = B_G_double->getColMap()->getGlobalElement(local_indices[k]);

                matrices->B_G->insertGlobalValues(global_index,
                                                  Teuchos::tuple<global_ordinal_type>(column_index),
                                                  Teuchos::tuple<scalar>((scalar)values[k]));
            }

            matrices->C->insertGlobalValues(global_index, Teuchos::tuple<global_ordinal_type>(global_index), Teuchos::tuple<scalar>((scalar)0.));
            matrices->Cb->insertGlobalValues(global_index, Teuchos::tuple<global_ordinal_type>(global_index), Teuchos::tuple<scalar>((scalar)1.));
        }
    }

    matrices->B->fillComplete(map->mapDofs, map->mapLagrangeDofs);
    matrices->B_2->fillComplete(map->mapDofs, map->mapLagrangeDofs);
    matrices->B_G->fillComplete(map->mapDofs, map->mapLagrangeDofs);
    matrices->B_T->fillComplete(map->mapLagrangeDofs, map->mapDofs);
    matrices->C->fillComplete();
    matrices->Cb->fillComplete();
};
#endif //KATOPTRON_MORTAR_H