/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WMSURFACE_H
#define WMSURFACE_H

#include "mirrors.h"
#include "wGroup.h"
#include <vector>
#include <string>
#include <Eigen/Dense>

using namespace tbox;

namespace mirrors
{

/**
 * @brief manage Mirrors surface
 */

class MIRRORS_API MSurface : public Group
{
public:
    std::string MS_name;
    std::vector<Node *> nodes;

    Eigen::Vector3d x0;
    Eigen::Vector3d x1;
    Eigen::Vector3d x2;

    MSurface(Problem &pbl, int no, std::string _MS_name, Eigen::Vector3d _x0, Eigen::Vector3d _x1, Eigen::Vector3d _x2);
    MSurface(Problem &pbl, std::string const &name, std::string _MS_name, Eigen::Vector3d _x0, Eigen::Vector3d _x1, Eigen::Vector3d _x2);

#ifndef SWIG
    virtual void write(std::ostream &out) const override;
#endif
};

} // namespace mirrors

#endif //WMSURFACE_H
