/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wMedium.h"
#include "wProblem.h"
#include "wTag.h"
using namespace mirrors;

Medium::Medium(Problem &pbl, int no, std::string _medium_name,
               double _E, double _nu, double _k, double _beta) : Group(pbl.msh, no), medium_name(_medium_name),
                                                                 E(_E), nu(_nu), k(_k), beta(_beta)
{
    pbl.media.push_back(this);
}

Medium::Medium(Problem &pbl, std::string const &name, std::string _medium_name,
               double _E, double _nu, double _k, double _beta) : Group(pbl.msh, name), medium_name(_medium_name),
                                                                 E(_E), nu(_nu), k(_k), beta(_beta)
{
    pbl.media.push_back(this);
}

void Medium::write(std::ostream &out) const
{
    out << "Medium " << medium_name << " on " << *tag
        << " with E=" << this->E << ", nu=" << this->nu
        << ", k=" << this->k << " and beta=" << this->beta;
}
