/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wSfHex8.h"

using namespace tbox;

void SfHex8::eval(Eigen::Vector3d const &ksi, Eigen::VectorXd &ff) const
{
    ff.resize(8);
    double x = ksi(0);
    double y = ksi(1);
    double z = ksi(2);

    ff(0) = 0.125 * (1.0 + x) * (1.0 + y) * (1.0 - z);
    ff(1) = 0.125 * (1.0 - x) * (1.0 + y) * (1.0 - z);
    ff(2) = 0.125 * (1.0 - x) * (1.0 - y) * (1.0 - z);
    ff(3) = 0.125 * (1.0 + x) * (1.0 - y) * (1.0 - z);
    ff(4) = 0.125 * (1.0 + x) * (1.0 + y) * (1.0 + z);
    ff(5) = 0.125 * (1.0 - x) * (1.0 + y) * (1.0 + z);
    ff(6) = 0.125 * (1.0 - x) * (1.0 - y) * (1.0 + z);
    ff(7) = 0.125 * (1.0 + x) * (1.0 - y) * (1.0 + z);
}

void SfHex8::evalD(Eigen::Vector3d const &ksi, Eigen::MatrixXd &dff) const
{
    dff.resize(3, 8);
    double x = ksi(0);
    double y = ksi(1);
    double z = ksi(2);

    dff(0, 0) = 0.125 * (1.0 + y) * (1.0 - z);
    dff(1, 0) = 0.125 * (1.0 + x) * (1.0 - z);
    dff(2, 0) = -0.125 * (1.0 + x) * (1.0 + y);
    dff(0, 1) = -0.125 * (1.0 + y) * (1.0 - z);
    dff(1, 1) = 0.125 * (1.0 - x) * (1.0 - z);
    dff(2, 1) = -0.125 * (1.0 - x) * (1.0 + y);
    dff(0, 2) = -0.125 * (1.0 - y) * (1.0 - z);
    dff(1, 2) = -0.125 * (1.0 - x) * (1.0 - z);
    dff(2, 2) = -0.125 * (1.0 - x) * (1.0 - y);
    dff(0, 3) = 0.125 * (1.0 - y) * (1.0 - z);
    dff(1, 3) = -0.125 * (1.0 + x) * (1.0 - z);
    dff(2, 3) = -0.125 * (1.0 + x) * (1.0 - y);
    dff(0, 4) = 0.125 * (1.0 + y) * (1.0 + z);
    dff(1, 4) = 0.125 * (1.0 + x) * (1.0 + z);
    dff(2, 4) = 0.125 * (1.0 + x) * (1.0 + y);
    dff(0, 5) = -0.125 * (1.0 + y) * (1.0 + z);
    dff(1, 5) = 0.125 * (1.0 - x) * (1.0 + z);
    dff(2, 5) = 0.125 * (1.0 - x) * (1.0 + y);
    dff(0, 6) = -0.125 * (1.0 - y) * (1.0 + z);
    dff(1, 6) = -0.125 * (1.0 - x) * (1.0 + z);
    dff(2, 6) = 0.125 * (1.0 - x) * (1.0 - y);
    dff(0, 7) = 0.125 * (1.0 - y) * (1.0 + z);
    dff(1, 7) = -0.125 * (1.0 + x) * (1.0 + z);
    dff(2, 7) = 0.125 * (1.0 + x) * (1.0 - y);
}
