/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WTETRA4_H
#define WTETRA4_H

#include "tbox.h"
#include "wLazy.h"
#include "wElement.h"
#include "wCacheTetra4.h"

namespace tbox
{

/**
 * @brief Linear tetrahedron finite element
 * @authors Kim Liegeois, Romain Boman, Adrien Crovato
 */
class TBOX_API Tetra4 : public Element
{
#ifndef SWIG
private:
    inline static CacheTetra4 &getCache(int n);

protected:
    virtual void buildJ() override;
    virtual void buildGradientJ() override;
    virtual void computeV() override;
    virtual void buildGradientV(int dim) override;
#endif

public:
    Tetra4(int n, Tag *_ptag, Tag *_etag, std::vector<Tag *> &_parts, std::vector<Node *> &nods);
    virtual ~Tetra4() {}
    virtual ELTYPE type() const override { return ELTYPE::TETRA4; }

#ifndef SWIG
    virtual void initValues(bool isvol, int ordr = 2) override;
    virtual void initGradients() override;
    virtual void update() override;
    virtual Cache &getVCache() const override;
    virtual Eigen::VectorXd computeGradient(std::vector<double> const &u, size_t k) const override;
#endif
};

#include "wTetra4.inl"

} // namespace tbox

#endif //WTETRA4_H
